local L = LibStub("AceLocale-3.0"):GetLocale("ShinyStatsClassic")
if not L then return end

ShinyStatsClassic = LibStub("AceAddon-3.0"):NewAddon("ShinyStatsClassic","AceConsole-3.0")
local _G = _G;



local SHINY_COLOR_ITEMLEVEL = "ilvl"
local SHINY_COLOR_HEALTH = "hp"
local SHINY_COLOR_AGILITY = "agi"
local SHINY_COLOR_VERSATILITY = "vers"
local SHINY_COLOR_STAMINA = "sta"
local SHINY_COLOR_SPIRIT = "spi"
local SHINY_COLOR_INTELLECT = "int"
local SHINY_COLOR_STRENGTH = "str"
local SHINY_COLOR_ATTACKPOWER = "ap"
local SHINY_COLOR_EXPERTISE = "exp"
local SHINY_COLOR_HASTE = "has"
local SHINY_COLOR_HIT = "hit"
local SHINY_COLOR_CRITICAL = "cri"
local SHINY_COLOR_ARMOR = "arm"
local SHINY_COLOR_PARRY = "par"
local SHINY_COLOR_BLOCK = "blo"
local SHINY_COLOR_DODGE = "dod"
local SHINY_COLOR_MASTERY = "mr"
local SHINY_COLOR_SPELLPOWER = "sp"
local SHINY_COLOR_SPELLPENETRATION = "spen"
local SHINY_COLOR_PVPPOWER = "pvp"
local SHINY_COLOR_PVPRESILIENCE = "res"
local SHINY_COLOR_FIRERESISTANCE = "fr"
local SHINY_COLOR_FROSTRESISTANCE = "frr"
local SHINY_COLOR_NATURERESISTANCE = "nr"
local SHINY_COLOR_SHADOWRESISTANCE = "sr"
local SHINY_COLOR_ARCANERESISTANCE = "ar"
local SHINY_COLOR_WEAPONDPS = "dps"
local SHINY_COLOR_WEAPONDAMAGE = "dmg"
local SHINY_COLOR_WEAPONSPEED = "spd"
local SHINY_COLOR_ENCHANT = "ench"
local SHINY_COLOR_PASSTHRU = "passthru"



function ShinyStatsClassic:GetOptions()
	local options = {
	--	name = "ShinyStatsClassic",
		handler = ShinyStatsClassic,
		type = 'group',
		args = {
					aceconfig = {
						name = L["ShinyStatsClassic settings (Ace style)"],
						desc = L["Shows the Ace configuration dialog for ShinyStatsClassic."],
						type = 'execute',
						func = function() LibStub("AceConfigDialog-3.0"):Open("ShinyStatsClassic") end,
						hidden = true,
						cmdHidden = false,
					},
					config = {
						name = L["ShinyStatsClassic settings"],
						desc = L["Shows the blizzard configuration dialog for ShinyStatsClassic."],
						type = 'execute',
						func = "ShinyConfig",
						hidden = true,
						cmdHidden = false,
					},
			general = {
				name = "General Settings",
				type = 'group',
				args = {
					--[[
					reset = {
						name = L["Reset"],
						desc = L["Reset settings back to defaults."],
						type = 'execute',
						func = function() ShinyStatsClassic.db:ResetProfile() end,
					}, --]]
					shortenEnchants = {
						type = 'toggle',
						name = L["Shorten Enchants"],
						desc = L["Whether or not you wish for the addon to reduce enchant/socket texts to a minimum."],
						set = 'SetOption',
						get = 'GetOption',
					},
					prefixEnchants = {
						type = 'toggle',
						name = L["Enchant Prefixing"],
						desc = L["Whether or not you wish for the addon to prefix enchants."],
						set = 'SetOption',
						get = 'GetOption',
					},
					shortenEquips = {
						type = 'toggle',
						name = L["Shorten Equips"],
						desc = L["Whether or not you wish for the addon to reduce equip texts to a minimum."],
						disabled = true,
						set = 'SetOption',
						get = 'GetOption',
					},		
					shortenSeparator = {
						type = 'input',
						name = L["Shorten Separator"],
						desc = L["The string replacing substituted values. (Does nothing if ShortenEquips and ShortenEnchants are disabled)"],
						set = 'SetOption',
						get = 'GetOption',
					},
					enablePartialEquip = {
						type = 'toggle',
						name = L["Enable Partial Equip Matching"],
						desc = L["Enables matching partial stats in Equip/Use statements (such as increases Spirit and Strength by 10). May not play nice together with RatingBuster."],
						set = 'SetOption',
						get = 'GetOption',
					},
					disableItemLevelColoring = {
						type = 'toggle',
						name = L["RatingBuster compatibility"],
						desc = L["Enable this if you are using RatingBuster to color Item Level and/or display Item ID."],
						set = 'SetOption',
						get = 'GetOption',
					},
				}
			},
			colors = {
						type = 'group',
						name = L["Colors"],
						desc = L["Set the colors of the various item stats."],
						args = {}
					},
			profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db),
		},
	}
	return options
end
	
function ShinyStatsClassic:ShinyConfig()
	InterfaceOptionsFrame_OpenToCategory("ShinyStatsClassic")
	InterfaceOptionsFrame_OpenToCategory("ShinyStatsClassic")
end

local defaults = {
	profile = {
		shortenEnchants = true,
		shortenEquips	= true,
		shortenSeparator = ", ",
		prefixEnchants 	= true,
		enablePartialEquip = false,
		disableItemLevelColoring = false,
		colors = {
			["hp"] = "85FF85",    -- Health

			["agi"] = "ffee03", 	-- Agility
			["sta"] = "dda357", 	-- Stamina
			["spi"] = "b7ffb8", 	-- Spirit
			["int"] = "b395ff", 	-- Intellect
			["str"] = "ff000a", 	-- Strength
			["vers"] = "b7aa88", 	-- Versatility
			
			["exp"] = "bbff00", 	-- Expertise
			["hit"] = "00ffc7", 	-- Hit Rating
			["cri"] = "e8850d", 	-- Crit Rating
			["has"] = "b3eeff", 	-- Haste Rating
			["ap"] = "ff494f", 		-- Attack Power
			
			["frr"] = "188bff", 	-- Frost Res
			["fr"] = "ff8300", 		-- Fire Res
			["nr"] = "87ff3c", 		-- Nature Res
			["sr"] = "bd33b1", 		-- Shadow Res
			["ar"] = "a0f2ff", 		-- Arcane Res
			
			["ilvl"] = "F2C038", 	-- Item Level
			
			["blo"] = "a2793c", 	-- Block
			["par"] = "f600ff", 	-- Parry
			["dod"] = "bcff00", 	-- Dodge

			["pvp"] = "686eff",		-- PvP Power
			["res"] = "686eff",		-- PvP Resilience
			
			["dmg"] = "ff8930", 	-- Damage
			["dps"] = "ff96ad", 	-- DPS
			["spd"] = "fff250",		-- Speed
			["arm"] = "b9b9b9", 	-- Armor
			
			["sp"]	= "ff73ba",		-- Spell Power
			["spen"] = "ffffff",	-- Spell Penetration
			
			["mr"]	= "ff73ba",		-- Mastery Rating
			
			-- Specials
			["ench"] = "00ff00",	-- Ench: Color
			["passthru"] = "passthru",	-- Ench: Color
			
			-- Unspecified
			['*']	 = "ffffff",	-- Default for unspecified
		},
	}
}


-- Bind colors to more descriptive names for options (Localization)
local colorNames = {
	-- Option Stat names
	["hp"] = L["Health"],       -- Health

	["agi"] = L["Agility"],			-- Agility
	["sta"] = L["Stamina"],			-- Stamina
	["spi"] = L["Spirit"],			-- Spirit
	["int"] = L["Intellect"],			-- Intellect
	["str"] = L["Strength"],			-- Strength
	["vers"] = L["Versatility"],		

	["exp"] = L["Expertise Rating"],			-- Expertise
	["cri"] = L["Critical Strike"],			-- Crit Rating
	["hit"] = L["Hit Rating"],			-- Hit Rating
	["has"] = L["Haste Rating"],			-- Haste Rating
	["ap"] = L["Attack Power"],			-- Attack Power

	["frr"] = L["Frost Resistance"],			-- Frost Res
	["fr"] = L["Fire Resistance"],			-- Fire Res
	["nr"] = L["Nature Resistance"],			-- Nature Res
	["sr"] = L["Shadow Resistance"],			-- Shadow Res
	["ar"] = L["Arcane Resistance"],			-- Arcane Res

	["ilvl"] = L["Item Level"],			-- Item Level

	["blo"] = L["Block Rating"],			-- Block
	["par"] = L["Parry Rating"],			-- Parry
	["dod"] = L["Dodge Rating"],			-- Dodge

	["pvp"] = L["PvP Power"],			-- Resilience
	["res"] = L["PvP Resilience"],			-- Resilience

	["dmg"] = L["Weapon Damage"],			-- Damage
	["dps"] = L["Weapon DPS"],			-- DPS
	["spd"] = L["Weapon Speed"],			-- Speed
	["arm"] = L["Armor"],				-- Armor

	["sp"] = L["Spell Power"],			-- Spell Power
	["spen"] = L["Spell Penetration"],		-- Spell Penetration
	
	["mr"] = L["Mastery Rating"],			-- Mastery Rating

	-- Specials
	["ench"] = L["Enchant Color"],			-- Ench: Color
}


function ShinyStatsClassic:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New("ShinyStatsClassicDB", defaults, true)
	
	self.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig")
	
	-- Init special variables
	self.recipeCalls = 0
	self.previousItemLink = nil
	
	-- Get Option Tables
	self.options = self:GetOptions()
	
	ShinyStatsClassic:RefreshConfig()

	self.debug = false
	
	for k,v in pairs(self.db.profile.colors) do
		if (k ~= "passthru") then
			self.options.args.colors.args[k] = {
				type = 'color',
				name = colorNames[k],
				set = 'SetOption',
				get = 'GetOption', 
			}
		end
	end
	
	-- Register settings
	local AceConfig = LibStub("AceConfig-3.0")
	local AceConfigDialog = LibStub("AceConfigDialog-3.0")
	
	AceConfig:RegisterOptionsTable("ShinyStatsClassic", self.options, {"ShinyStatsClassic"})
	self:RegisterChatCommand("shiny", "ChatCommand")
	
	BlizPanel = AceConfigDialog:AddToBlizOptions("ShinyStatsClassic", nil, nil, "general")
	AceConfigDialog:AddToBlizOptions("ShinyStatsClassic", "Colors","ShinyStatsClassic","colors")
	AceConfigDialog:AddToBlizOptions("ShinyStatsClassic", "Profiles","ShinyStatsClassic","profile")
	
	HookTooltips()
end


function ShinyStatsClassic:ChatCommand(userInput)
	if (userInput == "dbg" or userInput == "debug") then
		self.debug = not self.debug
	else
		self:ShinyConfig()
	end
end

function ShinyStatsClassic:SetOption(info,value, ...)
	local statements = {
		shortenEnchants	= function() self.db.profile.shortenEnchants = value end,
		shortenEquips	= function() self.db.profile.shortenEquips = value end,
		shortenSeparator = function() self.db.profile.shortenSeparator = gsub(value,"[\"\']","") end,
		prefixEnchants 	= function() self.db.profile.prefixEnchants = value end,
		enablePartialEquip = function() self.db.profile.enablePartialEquip = value end,
		disableItemLevelColoring = function() self.db.profile.disableItemLevelColoring = value end
	}
	if info[#info-1] == "colors" and type(value) == "number" then
		local r,g,b,a = value, ...
		self.db.profile.colors[info[#info]] = string.format("%02x%02x%02x",r*255,g*255,b*255)
		self.updateRequired = true
	elseif statements[info[#info]] then
		statements[info[#info]]()
		--print(format("ShinyStatsClassic: %s set to %s!",info[#info], tostring(value)))
	end
end

function ShinyStatsClassic:Hex2Dec(hex)
	local dec,i = {}, 0
	for v in string.gmatch(hex,"(%x%x)") do
		i=i+1
		dec[i] = tonumber(v,16)/255
	end
	return dec[1],dec[2],dec[3]
end


function ShinyStatsClassic:GetOption(info)
	if info[#info-1] == "colors" then
		local r,g,b = ShinyStatsClassic:Hex2Dec(self.db.profile.colors[info[#info]])
		return r,g,b,1
	else
		return self.db.profile[info[#info]]
	end
end

function ShinyStatsClassic:RefreshConfig()
	-- Bind patterns to colors
	local pColors = self.db.profile.colors

	self.stats = nil
  	self.stats = {
		{ L["%+?[%d,%%.]+ Health"],                                SHINY_COLOR_HEALTH },													-- Health

		{ L["%+?[%d,%%.]+ Agility"],                               SHINY_COLOR_AGILITY },													-- Agility
		{ L["Agility by [%d,%%.]+"],                               SHINY_COLOR_AGILITY },													-- Agility
		
		{ L["%+?[%d,%%.]+ Stamina"],                               SHINY_COLOR_STAMINA },													-- Stamina
		{ L["Stamina by [%d,%%.]+"],                               SHINY_COLOR_STAMINA },												-- Stamina
		
		{ L["%+?[%d,%%.]+ Spirit"],                                SHINY_COLOR_SPIRIT },													-- Spirit
		{ L["Spirit by [%d,%%.]+"],                                SHINY_COLOR_SPIRIT },													-- Spirit
		
		{ L["%+?[%d,%%.]+ Intellect"],                             SHINY_COLOR_INTELLECT },													-- Intellect
		{ L["Intellect by [%d,%%.]+"],                             SHINY_COLOR_INTELLECT },												-- Intellect
		
		{ L["%+?[%d,%%.]+ Strength"],                              SHINY_COLOR_STRENGTH },													-- Strength
		{ L["Strength by [%d,%%.]+"],                              SHINY_COLOR_STRENGTH },												-- Strength
		
		{ L["%+?[%d,%%.]+ Versatility"],                           SHINY_COLOR_VERSATILITY }, 		-- Versatility
		{ L["Versatility by [%d,%%.]+"],                           SHINY_COLOR_VERSATILITY },												-- Versatility
		
		{ L["%+?[%d,%%.]+ [Aa]ttack [Pp]ower"],                    SHINY_COLOR_ATTACKPOWER },													-- Attack Power
		{ L["%+?[%d,%%.]+ AP"],                                    SHINY_COLOR_ATTACKPOWER },													-- Attack Power
		{ L["Equip: Increases attack power by [%d,%%.]+"],         SHINY_COLOR_ATTACKPOWER },								-- Attack Power
		{ L["attack power by [%d,%%.]+"],                          SHINY_COLOR_ATTACKPOWER },												-- Attack Power		
						
		{ L["Equip: Increases your expertise by [%d,%%.]+"],       SHINY_COLOR_EXPERTISE },					-- Expertise
		{ L["expertise by [%d,%%.]+"],                             SHINY_COLOR_EXPERTISE },										-- Expertise
		{ L["%+?[%d,%%.]+ Expertise"],                             SHINY_COLOR_EXPERTISE },											-- Expertise					
								
		{ L["Equip: Increases your haste by [%d,%%.]+"],           SHINY_COLOR_HASTE },						-- Haste
		{ L["haste by [%d,%%.]+"],                                 SHINY_COLOR_HASTE },											-- Haste
		{ L["%+?[%d,%%.]+ [hH]aste"],                              SHINY_COLOR_HASTE },												-- Haste
						
		{ L["Equip: Increases your hit by [%d,%%.]+"],             SHINY_COLOR_HIT },						-- Hit	
		{ L["hit by [%d,%%.]+"],                                   SHINY_COLOR_HIT },												-- Hit
		{ L["%+?[%d,%%.]+ Hit"],                                   SHINY_COLOR_HIT },												-- Hit						
						
		{ L["Equip: Increases your critical strike by [%d,%%.]+"], SHINY_COLOR_CRITICAL },			-- Crit	
		{ L["critical strike by [%d,%%.]+"],                       SHINY_COLOR_CRITICAL },									-- Crit	
		{ L["%+?[%d,%%.]+ Critical Strike"],                       SHINY_COLOR_CRITICAL },									-- Crit
		{ L["%+?[%d,%%.]+ Crit"],                                  SHINY_COLOR_CRITICAL },												-- Crit

		{ L["Item Level %d*"],                                     SHINY_COLOR_ITEMLEVEL },													-- Item Level
						
		{ L["%+?[%d,%%.]+ Armor"],                                 SHINY_COLOR_ARMOR },														-- Armor
		{ L["%+?[%d,%%.]+ Physical Reduction"],                    SHINY_COLOR_ARMOR },														-- Physical Reduction

		{ L["Equip: Increases your parry by [%d,%%.]+"],           SHINY_COLOR_PARRY },						-- Parry
		{ L["parry by [%d,%%.]+"],                                 SHINY_COLOR_PARRY },												-- Parry
		{ L["%+?[%d,%%.]+ Parry"],                                 SHINY_COLOR_PARRY },												-- Parry

		{ L["Equip: Increases your block by [%d,%%.]+"],           SHINY_COLOR_BLOCK },						-- Block
		{ L["block by [%d,%%.]+"],                                 SHINY_COLOR_BLOCK },												-- Block	
		{ L["%+?[%d,%%.]+ Block"],                                 SHINY_COLOR_BLOCK },														-- Block

		{ L["Equip: Increases your dodge by [%d,%%.]+"],           SHINY_COLOR_DODGE },						-- Dodge
		{ L["dodge by [%d,%%.]+"],                                 SHINY_COLOR_DODGE },												-- Dodge		
		{ L["%+?[%d,%%.]+ Dodge"],                                 SHINY_COLOR_DODGE },												-- Dodge
			
		{ L["Equip: Increases your pvp power by [%d,%%.]+"],       SHINY_COLOR_PVPPOWER },				-- PVP Power

		{ L["Equip: Increases your pvp resilience by [%d,%%.]+"],  SHINY_COLOR_PVPRESILIENCE },				-- Resilience
		{ L["pvp resilience by [%d,%%.]+"],                        SHINY_COLOR_PVPRESILIENCE },											-- Resilience	
		{ L["%+?[%d,%%.]+ PvP Resilience"],                        SHINY_COLOR_PVPRESILIENCE },											-- Resilience

		{ L["%+?[%d,%%.]+ Fire Resistance"],                       SHINY_COLOR_FIRERESISTANCE },												-- Fire Res
		{ L["%+?[%d,%%.]+ Frost Resistance"],                      SHINY_COLOR_FROSTRESISTANCE },											-- Frost Res
		{ L["%+?[%d,%%.]+ Nature Resistance"],                     SHINY_COLOR_NATURERESISTANCE },												-- Nature Res
		{ L["%+?[%d,%%.]+ Shadow Resistance"],                     SHINY_COLOR_SHADOWRESISTANCE },												-- Shadow Res
		{ L["%+?[%d,%%.]+ Arcane Resistance"],                     SHINY_COLOR_ARCANERESISTANCE },												-- Arcane Res
						
		{ L["Equip: Increases spell power by [%d,%%.]+"],          SHINY_COLOR_SPELLPOWER },								-- Spell Power
		{ L["spell power by [%d,%%.]+"],                           SHINY_COLOR_SPELLPOWER },												-- Spell Power
		{ L["%+?[%d,%%.]+ Spell Power"],                           SHINY_COLOR_SPELLPOWER },													-- Spell Power
		
		{ L["Equip: Increases spell penetration by [%d,%%.]+"],    SHINY_COLOR_SPELLPENETRATION },						-- Spell Penetration
		{ L["spell penetration by [%d,%%.]+"],                     SHINY_COLOR_SPELLPENETRATION },										-- Spell Penetration
		{ L["%+?[%d,%%.]+ Spell Penetration"],                     SHINY_COLOR_SPELLPENETRATION },											-- Spell Penetration
						
		{ L["%([%d,%%.]+ damage per second%)"],                    SHINY_COLOR_WEAPONDPS },									-- DPS

		{ L["[%d,%-%s]+ Damage"],                                  SHINY_COLOR_WEAPONDAMAGE },												-- Damage
		
		{ L["Speed [%d,%%.]+"],                                    SHINY_COLOR_WEAPONSPEED },													-- Speed

		{ L["Equip: Increases your mastery by [%d,%%.]+"],         SHINY_COLOR_MASTERY },						-- Mastery Rating
		{ L["mastery by [%d,%%.]+"],                               SHINY_COLOR_MASTERY },										-- Mastery Rating
		{ L["%+?[%d,%%.]+ Mastery"],                               SHINY_COLOR_MASTERY },												-- Mastery Rating

		{ L["^Binds when picked up"],                             SHINY_COLOR_PASSTHRU },													-- Equip String
		{ L["^Binds on equip"],                                   SHINY_COLOR_PASSTHRU },													-- Equip String
		{ L["^Soulbound"],                                        SHINY_COLOR_PASSTHRU },													-- Equip String
		{ L["^Unique"],                                           SHINY_COLOR_PASSTHRU },													-- Equip String
		{ L["^Unique-Equipped"],                                  SHINY_COLOR_PASSTHRU },													-- Equip String
		{ L["^Enchanted"],                                        SHINY_COLOR_PASSTHRU }, 													-- Equip String
		{ L["^Legacy Set"],                                       SHINY_COLOR_PASSTHRU },												-- Legacy Set String
		{ L["^Chance on hit"],                                    SHINY_COLOR_PASSTHRU }, 													-- Proc (Chance on hit)
		{ L["%([%d,.]+ Min Cooldown%)"],                          SHINY_COLOR_PASSTHRU }, 													-- On Use Cooldown

		{ L["%+?[%d,%%.]+ PvP Power"],                            SHINY_COLOR_PVPPOWER }, 		-- PVP POWER

		{ L["Equip: Armor increased by %+?[%d,%%.]+"],            SHINY_COLOR_ARMOR },											-- Armor
		
	}

	self.updateRequired = false
end

function ShinyStatsClassic:ExtendTooltip(tip, itemLink)
		local config = self.db.profile
		local colors = self.db.profile.colors

		-- Work around in order to avoid fucking up tooltips with embedded items (such as patterns/recipes)
		-- Seeing such tooltips are called twice, this checks if it has already been called, if it has, dont process it a second time
		if (string.match(itemLink,L["Recipe"]) or string.match(itemLink,L["Pattern"]) or string.match(itemLink,L["Plans"]) or string.match(itemLink,L["Formula"]) or string.match(itemLink,L["Technique"]) or string.match(itemLink,L["Design"]) or string.match(itemLink,L["Manual"])) then 
			if  self.recipeCalls > 0 then
				self.recipeCalls = 0
				return
			else
				self.recipeCalls = 1
			end
		end
		
		--print("ShinyStatsClassic: ExtendTooltip called!");
		if self.updateRequired then ShinyStatsClassic:RefreshConfig() end
		local regions = {tip:GetRegions()}
		for i = 1, #regions do
			local region = regions[i]
			if region and region:GetObjectType() == "FontString" then
				local r,g = region:GetTextColor()
				local text = region:GetText()
				local origText = text

				if (text and self.debug) then
					print("ShinyStatsClassic Debug: " .. text)
				end

				if text then
					local partialMatch, fullMatch, matchFound, partialMatchFound, fullMatches, partialMatches = nil, nil, 0, 0, 0, 0

					--if string.match(text,L["^Equipment Sets"]) or string.match(text,L["^Equipment Sets"])
					
					-- Check if Socket bonus is activated (also check if you have reached the end part where no modification will be needed, then "continue")
					if not ((string.match(text,L["^Socket Bonus"]) and r < 0.51 and g < 0.51) or string.match(text,L["^<Shift"]) or string.match(text,L["^<Made"]) or string.match(text,L["^<Right"]) or string.match(text,L["^Reforged"]) or string.match(text,L["^Season"]) or (string.match(text,L["Item Level %d*"]) and self.db.profile.disableItemLevelColoring) or (string.match(text,L["^%(*%d*%)*%s*Set"]) and r < 0.51 and g < 0.51))  then
						-- Continue
						-- Do nothing, stupid LUA lacks continue statements and a break kills the main loop
					-- else
						-- Add "Ench: " to enchants / Munch and's
						-- if r < 0.05 and g > 0.95 and not string.match(text,L["^Socket"]) and not string.match(text,L["^Equip"]) and not string.match(text,L["^Chance"]) and not string.match(text,L["^%(*%d*%)*%s*Set"]) and not string.match(text,L["^Use"]) and not string.match(text,L["^Heroic"]) then
						
						-- 	-- Add "Ench: " to enchants
						-- 	if self.db.profile.prefixEnchants then
						-- 		text = format("|cff%s%s: %s|r",self.db.profile.colors.ench,L["Ench"],text)
						-- 	end
						
						-- 	-- Munch 'ands' and replace with choosen separator
						-- 	if self.db.profile.shortenEnchants then
						-- 		text = gsub(text,L["%sand%s"],self.db.profile.shortenSeparator)
						-- 	end
						-- 	if self.db.profile.shortenEquips then
						-- 		-- Do something later on in future
						-- 	end

						-- end
						if (text:match(L["^Enchanted"])) then
							-- TODO: Do something with the "Enchanted" prefix, change it, remove it, whatever.
							-- if config.prefixEnchants then
							-- 	text = format("|cff%s%s: %s|r", colors.ench, L["Ench"],text)
						end

						for i,v in ipairs(self.stats) do 
							
							local color = v[2] ~= SHINY_COLOR_PASSTHRU and self.db.profile.colors[v[2]] or false
							local pattern = v[1]
							--[[
							In order to avoid multiple matches for single line Equip in the german client (and possibly others),
							this check is neccessary. This makes sure only complete statements including the phrase "Equip" can match
							a statement starting with Equip. 
							
							For example;
							"Equip: Increases your critical strike rating by x" can match but "critical strike rating by x" can not.
							--]]
							
							-- Entire line matching
							if string.match(pattern,L["^Equip"]) and color then
								-- Perform matching against stats table
								-- Set the fullMatch and matchFound variables
								-- if (v[2] ~= colors.passthru) then
								fullMatch,matchFound = gsub(text,pattern,("|cff%s%s"):format(color,"%1"))
								-- end
								
								-- An entire match has been found if matchFound evalutes to true (more than 0 and not nil)
								if matchFound > 0 then
									fullMatches = fullMatches + matchFound
									break; -- Break the loop, entire line match found
								end
							-- Partial matching
							elseif (not string.match(origText,L["^Equip"]) and not string.match(origText,L["^Use"]) or self.db.profile.enablePartialEquip) and color then
								-- Check if partialMatches is not nil (meaning there might be more matches on a single line)
								-- Perform matching against stats table
								if partialMatch then
									textToMatch = partialMatch
									--print(partialMatch,text)
								else
									textToMatch = text
								end
								
								-- Only apply color if the color choosen is not nil
								-- if (self.debug) then print(text, pattern, color) end

								partialMatch,partialMatchFound = gsub(textToMatch,pattern,("|cff%s%s|r"):format(color,"%1"))
					

								if partialMatchFound > 0 then
									partialMatches = partialMatches + partialMatchFound
								end
							end
						end

						-- If full match found, set to full match, not set to partial matches, if any, else do nothing
						--print(text,matchFound,partialMatchFound)
						if fullMatches > 0 then
							--print("Trying to print full match",fullMatch)
							region:SetText(fullMatch)

						elseif partialMatches > 0 then
							--print("Trying to partial match",partialMatches)
							region:SetText(partialMatch)

						elseif (origText ~= text) then
							region:SetText(text)
						end
					end
				end
			end
		end
		tip:Show()
end

local function handleTooltipHook(tip, ...)
   local _, link = tip:GetItem()
   if link then
   		ShinyStatsClassic:ExtendTooltip(tip, link)
   end
end

-- local function hookCompareTip(tooltip)
--   if not tooltip then return end
--   hooksecurefunc(tooltip, 'OnTooltipSetItem', function(tip, mainLink)
--     local _, link = tip:GetItem()
--     if link then
--       ShinyStatsClassic:ExtendTooltip(tip, link)
--     end
--   end)
-- end

-- local function hookTipScript(tooltip)
--   if tooltip and tooltip.HookScript then
--     tooltip:HookScript('OnTooltipSetItem', function(tip, ...)
--       local _, link = tip:GetItem()
--       if link and GetItemInfo(link) then
--         ShinyStatsClassic:ExtendTooltip(tip, link)
--       end
--     end)
--   end
-- end

-- local hooked = false
-- function HookTooltips()
--   if hooked then return end
--   hooked = true

--   hookTipScript(GameTooltip)
--   hookTipScript(ItemRefTooltip)

--   hookCompareTip(ShoppingTooltip1)
--   hookCompareTip(ShoppingTooltip2)

--   hookCompareTip(ItemRefShoppingTooltip1)
--   hookCompareTip(ItemRefShoppingTooltip2)

--   if LinkWrangler and LinkWrangler.RegisterCallback then
--     LinkWrangler.RegisterCallback("ShinyStatsClassic", function(frame,link)
--       ShinyStatsClassic:ExtendTooltip(frame,link)
--     end, "item")
--   end

-- end

local hooked = false
function HookTooltips()
  if hooked then return end
  hooked = true

  GameTooltip:HookScript("OnTooltipSetItem", handleTooltipHook)
  ItemRefTooltip:HookScript("OnTooltipSetItem", handleTooltipHook)

  ItemRefShoppingTooltip1:HookScript("OnTooltipSetItem", handleTooltipHook)
  ItemRefShoppingTooltip2:HookScript("OnTooltipSetItem", handleTooltipHook)

  ShoppingTooltip1:HookScript("OnTooltipSetItem", handleTooltipHook)
  -- hooksecurefunc(ShoppingTooltip1, "AdvanceSecondaryCompareItem", function(...) print(...) end)

  ShoppingTooltip2:HookScript("OnTooltipSetItem", handleTooltipHook)
  -- ShoppingTooltip2:HookScript("OnTooltipSetItem", handleTooltipHook)

  if LinkWrangler and LinkWrangler.RegisterCallback then
    LinkWrangler.RegisterCallback("ShinyStatsClassic", function(frame,link)
      ShinyStatsClassic:ExtendTooltip(frame,link)
    end, "item")
  end

end





-- The following stuff is unfortunately necessary, because some addons *override*
-- the tooltip handlers instead of hooking them :(

local f = CreateFrame("Frame")
local entered = false
local elapsed = 0
local timerStarted = false
f:SetScript("OnEvent", function(f, e, ...) f[e](f, ...) end)

function f:PLAYER_ENTERING_WORLD()
	entered = true
	f:ScheduleHook()
end

function f:ADDON_LOADED()
	if entered then
		f:ScheduleHook()
	end
end

function f:ScheduleHook()
	if timerStarted then
		elapsed = 0
	else
		f:SetScript("OnUpdate", f.OnTimer)
	end
end

function f:OnTimer(eps)
	elapsed = elapsed + eps
	if elapsed >= 3 then
		f:SetScript("OnUpdate", nil)
		HookTooltips()
	end
end

f:RegisterEvent("PLAYER_ENTERING_WORLD")
f:RegisterEvent("ADDON_LOADED")